import 'dart:developer' as developer;

typedef LooperCallback<T> = void Function(T data, int index, bool isFirst, bool isLast);
typedef AsyncTask<T> = Future<T> Function();
typedef AsyncVoidTask = void Function();

loop<T>(List<T> items, LooperCallback<T> callback){
  NUICodeUtil.loop(items, callback);
}

T getIfAvailable<T>(List<T> items, int position){
  if(items.length > position){
    return items[position];
  }
  return null;
}

void logNUI(String module, String content){
  developer.log("[$module] $content");
}

void throwNUI(String module, String description){
  throw Exception("[$module] $description");
}

class NUIAsync{
  NUIAsync._builder();

  static Future<T> run<T>(AsyncTask<T> task) async{
    return task();
  }

  static void delay(Duration duration, AsyncVoidTask task) async{
    Future.delayed(duration, (){
      return task();
    });
  }

  static Future<T> delayForResult<T>(Duration duration, AsyncTask<T> task) async{
    return Future.delayed(duration, (){
      return task();
    });
  }
}
class NUICodeUtil{
  static void loop<T>(List<T> items, LooperCallback<T> callback){
    int index = 0;
    for(T item in items){
      callback(item, index, index == 0, index == items.length - 1);
      index++;
    }
  }
}
